/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * /*
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gui.Airfoil;

import main.airfoil.ConstantAirfoil;
import gui.Utils.EntryReal;
import java.awt.Font;
import java.util.Properties;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import net.miginfocom.swing.MigLayout;

/**
 *
 * @author J. Fourcade
 */
public class ConstantSlopeAirfoilGUI extends JPanel {

    // Datas
    JTextField a, delta, momentCoefficient, aerodynamicCenterLocation;
    Font font ;

    /**
     *
     * @param airfoilProfile
     */
    public ConstantSlopeAirfoilGUI(ConstantAirfoil airfoilProfile) {

        // Layout
        setLayout(new MigLayout("insets 5 5 5 5", "[]15[]15[]15[]"));
        
        // Panel font and title font
        setFont(new Font("Courier", Font.PLAIN, 12)) ;
        font = getFont().deriveFont(Font.BOLD,16) ;

        // Title
        JLabel title = new JLabel("Airfoil :");
        title.setFont(font);
        add(title, "wrap, spanx 10");

        // LiftSlope
        add(new JLabel("Lift slope"));
        add(new JLabel(" a"));
        a = new JTextField(Double.toString(airfoilProfile.getLiftSlope()));
        add(a, "w 80");

        // Profile drag
        add(new JLabel("Profile drag"));
        add(new JLabel(" δ"));
        delta = new JTextField(Double.toString(airfoilProfile.getProfileDrag()));
        add(delta, "w 80, wrap");
        
        // Moment coefficient
        add(new JLabel("Moment coefficient"));
        add(new JLabel(" Cm"));
        momentCoefficient = new JTextField(Double.toString(
                airfoilProfile.getMomentCoefficient()));
        add(momentCoefficient, "w 80");
        
        // Aerodynamic center location
        add(new JLabel("Aerodynamic center location"));
        add(new JLabel(" xac"));
        aerodynamicCenterLocation = new JTextField(Double.toString(
                airfoilProfile.getAerodynamicCenterLocation()));
        add(aerodynamicCenterLocation, "w 80, wrap");
    }

    /**
     *
     * @param prop
     */
    public void read(Properties prop) {
        EntryReal.readProperties(prop, a, "Airfoil.Lift_slope");
        EntryReal.readProperties(prop, delta, "Airfoil.Profile_drag");
        EntryReal.readProperties(prop, momentCoefficient, "Airfoil.Moment_coefficient");
        EntryReal.readProperties(prop, aerodynamicCenterLocation, 
                "Airfoil.Aerodynamic_center_location");
    }

    /**
     *
     * @param prop
     */
    public void write(Properties prop) {
        EntryReal.writeProperties(prop, a, "Airfoil.Lift_slope");
        EntryReal.writeProperties(prop, delta, "Airfoil.Profile_drag");
        EntryReal.writeProperties(prop, momentCoefficient, "Airfoil.Moment_coefficient");
        EntryReal.writeProperties(prop, aerodynamicCenterLocation, 
                "Airfoil.Aerodynamic_center_location");
    }

    /**
     *
     * @return
     */
    public ConstantAirfoil getAirfoilProfile() {
        final double aValue = EntryReal.read(a);
        if (aValue <= 0) {
            throw new IllegalArgumentException("Airfoil lift slope must be strictyl positive");
        }
        final double deltaValue = EntryReal.read(delta);
        if (deltaValue <= 0) {
             throw new IllegalArgumentException("Airfoil profile drag must be strictyl positive");
        }
        final double momentCoefficientValue = EntryReal.read(momentCoefficient);
        final double aerodynamicCenterLocationValue = EntryReal.read(aerodynamicCenterLocation);
        return new ConstantAirfoil(aValue, deltaValue,momentCoefficientValue,
                aerodynamicCenterLocationValue);
    }
}
