/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * /*
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gui.Atmosphere;

import gui.Utils.UnitDependant;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import main.atmosphere.Atmosphere;
import main.atmosphere.ConstantAtmosphere;
import main.atmosphere.DensityAtmosphere;
import main.atmosphere.StandardAtmosphere;
import net.miginfocom.swing.MigLayout;

/**
 *
 * @author J. Fourcade
 */
public class AtmosphereGUI extends JPanel implements UnitDependant {

    // Datas
    int mode;
    JRadioButton rb1, rb2;
    JTextField a, delta;
    StandardAtmosphereGUI standardAtmosphereGUI;
    DensityAtmosphereGUI densityAtmosphereGUI;
    ConstantAtmosphereGUI constantAtmosphereGUI;
    Font font;

    class RadioListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            if (e.getActionCommand().equals("Standard")) {
                standardAtmosphereGUI.setVisible(true);
                if (mode == 0) {
                    densityAtmosphereGUI.setVisible(false);
                } else {
                    standardAtmosphereGUI.setVisible(false);
                }
            } else {
                standardAtmosphereGUI.setVisible(false);
                if (mode == 0) {
                    densityAtmosphereGUI.setVisible(true);
                } else {
                    standardAtmosphereGUI.setVisible(true);
                }
            }
        }
    }

    /**
     *
     * @param atmosphere
     * @param mode = 0 DensityAtmospher, = 1 ConstantAtmosphere
     */
    public AtmosphereGUI(Atmosphere atmosphere, int mode) {

        // Layout
        setLayout(new MigLayout("hidemode 3, insets 5 5 5 5"));

        // Panel font and title font
        setFont(new Font("Courier", Font.PLAIN, 12));
        font = getFont().deriveFont(Font.BOLD, 16);

        // Title
        JLabel title = new JLabel("Atmosphere :");
        title.setFont(font);
        add(title, "wrap");

        // Atmosphere model choice
        rb1 = new JRadioButton("Standard Atmosphere");
        rb1.setSelected(true);
        add(rb1, "split");
        rb1.setActionCommand("Standard");
        if (mode == 0) {
            rb2 = new JRadioButton("Density Atmosphere");
        } else {
            rb2 = new JRadioButton("Constant Atmosphere");
        }
        add(rb2, "wrap");

        // Group
        ButtonGroup group = new ButtonGroup();
        group.add(rb1);
        group.add(rb2);

        // Listener
        RadioListener myListener = new RadioListener();
        rb1.addActionListener(myListener);
        rb2.addActionListener(myListener);

        // Test input data
        if (atmosphere instanceof StandardAtmosphere) {
            rb1.setSelected(true);
            rb2.setSelected(false);
            standardAtmosphereGUI = new StandardAtmosphereGUI(
                    (StandardAtmosphere) atmosphere);
            add(standardAtmosphereGUI, "wrap");
            if (mode == 0) {
                constantAtmosphereGUI = new ConstantAtmosphereGUI(
                        new ConstantAtmosphere(1.225, 15, 9.80665));

                add(constantAtmosphereGUI, "wrap");
                constantAtmosphereGUI.setVisible(false);
            } else {
                densityAtmosphereGUI = new DensityAtmosphereGUI(
                        new DensityAtmosphere(1.225));

                add(densityAtmosphereGUI, "wrap");
                densityAtmosphereGUI.setVisible(false);

            }
        } else {
            standardAtmosphereGUI = new StandardAtmosphereGUI(
                    new StandardAtmosphere(0, 0));
            add(standardAtmosphereGUI, "wrap");
            standardAtmosphereGUI.setVisible(false);
            if (atmosphere instanceof ConstantAtmosphere) {
                rb1.setSelected(false);
                rb2.setSelected(true);
                constantAtmosphereGUI = new ConstantAtmosphereGUI(
                        (ConstantAtmosphere) atmosphere);
                add(constantAtmosphereGUI, "wrap");
            }
            if (atmosphere instanceof DensityAtmosphere) {
                rb1.setSelected(false);
                rb2.setSelected(true);
                densityAtmosphereGUI = new DensityAtmosphereGUI(
                        (DensityAtmosphere) atmosphere);
                add(densityAtmosphereGUI, "wrap");
            }
        }
    }

    /**
     *
     */
    @Override
    public void unitsUpdate() {
        standardAtmosphereGUI.unitsUpdate();
        if (mode == 0) {
            densityAtmosphereGUI.unitsUpdate();
        } else {
            constantAtmosphereGUI.unitsUpdate();
        }
    }

    /**
     *
     * @param prop
     */
    public void read(Properties prop) {
        String s = prop.getProperty("Atmosphere.type");
        if (s.equalsIgnoreCase("Standard")) {
            rb1.setSelected(true);
            rb2.setSelected(false);
            standardAtmosphereGUI.read(prop);
            standardAtmosphereGUI.setVisible(true);
            if (mode == 0) {
                densityAtmosphereGUI.setVisible(false);
            } else {
                constantAtmosphereGUI.setVisible(false);
            }
        } else {
            rb1.setSelected(false);
            rb2.setSelected(true);
            standardAtmosphereGUI.setVisible(false);
            if (mode == 0) {
                densityAtmosphereGUI.setVisible(true);
                densityAtmosphereGUI.read(prop);
            } else {
                constantAtmosphereGUI.setVisible(true);
                constantAtmosphereGUI.read(prop);
            }
        }
    }

    /**
     *
     * @param prop
     */
    public void write(Properties prop) {
        if (rb1.isSelected()) {
            prop.setProperty("Atmosphere.type", "Standard");
            standardAtmosphereGUI.write(prop);
        } else {
            if (mode == 0) {
                prop.setProperty("Atmosphere.type", "Density");
                densityAtmosphereGUI.write(prop);
            } else {
                prop.setProperty("Atmosphere.type", "Constant");
                constantAtmosphereGUI.write(prop);
            }
        }
    }

    /**
     *
     * @return
     */
    public Atmosphere getAtmosphere() {
        if (rb1.isSelected()) {
            return standardAtmosphereGUI.getAtmosphere();
        } else {
            if (mode == 0) {
                return densityAtmosphereGUI.getAtmosphere();
            } else {
                return constantAtmosphereGUI.getAtmosphere();
            }
        }
    }
}
