/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * /*
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gui.Rotor;

import gui.Airfoil.ConstantSlopeAirfoilGUI;
import gui.InducedVelocity.InducedVelocityGUI;
import gui.Utils.EntryReal;
import gui.Utils.UnitDependant;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import main.rotor.GlauertRotor;
import main.utils.Units;
import net.miginfocom.swing.MigLayout;
import org.apache.commons.math3.util.FastMath;

/**
 *
 * @author J. Fourcade
 */
public class GlauertRotorGUI extends JPanel implements UnitDependant {

    // Airfoil profile datas
    private final ConstantSlopeAirfoilGUI airfoilGUI;
    // Induced velocity model datas
    private final InducedVelocityGUI inducedVelocityGUI;
    // Rotor datas
    private final JTextField b, R, c, xcg, thetaTW, GJ, B, Ib, Ic, ml;
    private final JLabel RUnit, cUnit, GJUnit, thetaTWUnit, IbLabel, IbName, IbUnit,
            IcLabel, IcName, IcUnit, mlLabel, mlName, mlUnit;
    JRadioButton rb1, rb2;

    class RadioListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            display(e.getActionCommand());
        }
    }

    private void display(String command) {
        if (command.equals("Entered")) {
            IbLabel.setVisible(true);
            IbName.setVisible(true);
            Ib.setVisible(true);
            IbUnit.setVisible(true);
            IcLabel.setVisible(true);
            IcName.setVisible(true);
            Ic.setVisible(true);
            IcUnit.setVisible(true);
            mlLabel.setVisible(false);
            mlName.setVisible(false);
            ml.setVisible(false);
            mlUnit.setVisible(false);
        } else {
            IbLabel.setVisible(false);
            IbName.setVisible(false);
            Ib.setVisible(false);
            IbUnit.setVisible(false);
            IcLabel.setVisible(false);
            IcName.setVisible(false);
            Ic.setVisible(false);
            IcUnit.setVisible(false);
            mlLabel.setVisible(true);
            mlName.setVisible(true);
            ml.setVisible(true);
            mlUnit.setVisible(true);
        }
    }

    /**
     *
     * @param rotor
     */
    public GlauertRotorGUI(GlauertRotor rotor) {

        // Layout
        setLayout(new MigLayout("insets 0 0 0 0"));

        // Panel font and title font
        setFont(new Font("Courier", Font.PLAIN, 12));
        Font font = getFont().deriveFont(Font.BOLD, 16);

        // Airfoil GUI
        airfoilGUI = new ConstantSlopeAirfoilGUI(rotor.getBladeAirfoil());

        // Induced velocity GUI
        inducedVelocityGUI = new InducedVelocityGUI(rotor.getInducedVelocity());

        // Rotor panel
        JPanel rotorPanel = new JPanel();
        rotorPanel.setLayout((new MigLayout("hidemode 3, insets 5 5 5 5",
                "[]15[]15[]5[]20[]15[]15[]5[]")));

        // Title
        JLabel title = new JLabel("Glauert Rotor :");
        title.setFont(font);
        rotorPanel.add(title, "wrap, spanx 10");

        // Number of blades
        rotorPanel.add(new JLabel("Number of blades"));
        rotorPanel.add(new JLabel(" b"));
        b = new JTextField(Integer.toString(rotor.getNumberOfBlade()));
        rotorPanel.add(b, "w 80, wrap");

        // Blade radius
        rotorPanel.add(new JLabel("Blade radius"));
        rotorPanel.add(new JLabel(" R"));
        R = new JTextField(String.format("%.2f",
                Units.convertValueFromSI(rotor.getBladeRadius(), Units.UnitsList.length1)));
        rotorPanel.add(R, "w 80");
        RUnit = new JLabel(Units.getLabel(Units.UnitsList.length1));
        rotorPanel.add(RUnit);

        // Blade chord
        rotorPanel.add(new JLabel("Blade chord"));
        rotorPanel.add(new JLabel(" c"));
        c = new JTextField(String.format("%.2f",
                Units.convertValueFromSI(rotor.getBladeChord(), Units.UnitsList.length2)));
        rotorPanel.add(c, "w 80");
        cUnit = new JLabel(Units.getLabel(Units.UnitsList.length2));
        rotorPanel.add(cUnit, "wrap");

        // Blade center of gravity location
        rotorPanel.add(new JLabel("Chordwise center of gravity location"));
        rotorPanel.add(new JLabel(" xcg"));
        xcg = new JTextField(String.format("%.2f", rotor.getBladeCenterOfGravityLocation()));
        rotorPanel.add(xcg, "w 80");

        // Blade pitch
        rotorPanel.add(new JLabel("Blade twist"), "skip 1");
        rotorPanel.add(new JLabel(" θTW"));
        thetaTW = new JTextField(String.format("%.2f",
                Units.convertValueFromSI(rotor.getBladeTwist(), Units.UnitsList.angle)));
        rotorPanel.add(thetaTW, "w 80");
        thetaTWUnit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        rotorPanel.add(thetaTWUnit, "wrap");

        // Blade torsional rigidity
        rotorPanel.add(new JLabel("Blade torsional rigidity"));
        rotorPanel.add(new JLabel(" GJ"));
        GJ = new JTextField(String.format("%.2f",
                Units.convertValueFromSI(rotor.getBladeTorsionalRigidity(),
                        Units.UnitsList.torsionalRigidity)));
        rotorPanel.add(GJ, "w 80");
        GJUnit = new JLabel(Units.getLabel(Units.UnitsList.torsionalRigidity));
        rotorPanel.add(GJUnit);

        // Tip loss factor
        rotorPanel.add(new JLabel("Tip loss factor"));
        rotorPanel.add(new JLabel(" B"));
        B = new JTextField(String.format("%.2f", rotor.getTipLossFactor()));
        rotorPanel.add(B, "w 80, wrap");

        // Menu for blade moment of inertia
        rotorPanel.add(new JLabel("Blade moment of inertia type"));
        rb1 = new JRadioButton("Entered by user");
        rotorPanel.add(rb1, "spanx 2");
        rb1.setActionCommand("Entered");
        rb2 = new JRadioButton("Computed");
        rotorPanel.add(rb2, "spanx 2, wrap");
        rb2.setActionCommand("Computed");

        // Group
        ButtonGroup group = new ButtonGroup();
        group.add(rb1);
        group.add(rb2);

        // Listener
        RadioListener myListener = new RadioListener();
        rb1.addActionListener(myListener);
        rb2.addActionListener(myListener);

        // Blade flapping moment of inertia
        IbLabel = new JLabel("Blade flapping moment of inertia");
        rotorPanel.add(IbLabel);
        IbName = new JLabel(" Ib");
        rotorPanel.add(IbName);
        Ib = new JTextField(String.format("%.4f",
                Units.convertValueFromSI(rotor.getBladeFlappingMomentOfInertia(),
                        Units.UnitsList.momentOfInertia)));
        rotorPanel.add(Ib, "w 80");
        IbUnit = new JLabel(Units.getLabel(Units.UnitsList.momentOfInertia));
        rotorPanel.add(IbUnit, "spanx 2, wrap");

        // Blade lagging moment of inertia
        IcLabel = new JLabel("Blade lagging moment of inertia");
        rotorPanel.add(IcLabel);
        IcName = new JLabel(" Ic");
        rotorPanel.add(IcName);
        Ic = new JTextField(String.format("%.4f",
                Units.convertValueFromSI(rotor.getBladeLaggingMomentOfInertia(),
                        Units.UnitsList.momentOfInertia)));
        rotorPanel.add(Ic, "w 80");
        IcUnit = new JLabel(Units.getLabel(Units.UnitsList.momentOfInertia));
        rotorPanel.add(IcUnit, "spanx 2, wrap");

        // Blade mass
        mlLabel = new JLabel("Blade mass per unit length");
        rotorPanel.add(mlLabel);
        mlName = new JLabel(" ml");
        rotorPanel.add(mlName);
        ml = new JTextField(String.format("%.2f",
                Units.convertValueFromSI(0, Units.UnitsList.massPerLength)));
        rotorPanel.add(ml, "w 80");
        mlUnit = new JLabel(Units.getLabel(Units.UnitsList.massPerLength));
        rotorPanel.add(mlUnit, "wrap");
        mlLabel.setVisible(false);
        mlName.setVisible(false);
        ml.setVisible(false);
        mlUnit.setVisible(false);

        // Initial state
        rb1.setSelected(true);
        rb2.setSelected(false);
        display("Entered");

        // Add components
        add(airfoilGUI, "wrap");
        add(inducedVelocityGUI, "wrap");
        add(rotorPanel, "wrap");
    }

    /**
     *
     */
    @Override
    public void unitsUpdate() {
        EntryReal.unitUpdate(R, "%.3f", RUnit, Units.UnitsList.length1);
        EntryReal.unitUpdate(c, "%.2f", cUnit, Units.UnitsList.length2);
        EntryReal.unitUpdate(ml, "%.3f", mlUnit, Units.UnitsList.massPerLength);
        EntryReal.unitUpdate(GJ, "%.2f", GJUnit, Units.UnitsList.torsionalRigidity);
        EntryReal.unitUpdate(Ib, "%.4f", IbUnit, Units.UnitsList.momentOfInertia);
        EntryReal.unitUpdate(Ic, "%.4f", IbUnit, Units.UnitsList.momentOfInertia);
    }

    /**
     *
     * @param prop
     */
    public void read(Properties prop) {
        airfoilGUI.read(prop);
        inducedVelocityGUI.read(prop);
        EntryReal.readProperties(prop, b, "GlauertRotor.Number_of_blades");
        EntryReal.readProperties(prop, R, "GlauertRotor.Blade_length");
        EntryReal.readProperties(prop, c, "GlauertRotor.Blade_chord");
        EntryReal.readProperties(prop, xcg, "GlauertRotor.Chordwise_center_of_gravity_location");
        EntryReal.readProperties(prop, thetaTW, "GlauertRotor.Blade_twist");
        EntryReal.readProperties(prop, GJ, "GlauertRotor.Torsional_Rigidity");
        EntryReal.readProperties(prop, B, "GlauertRotor.Tip_loss_factor");
        String s = prop.getProperty("GlauertRotor.Blade_moment_of_inertia_type");
        if (s.equalsIgnoreCase("Entered_by_user")) {
            rb1.setSelected(true);
            rb2.setSelected(false);
            display("Entered");
            EntryReal.readProperties(prop, Ib, "GlauertRotor.Blade_flapping_moment_of_inertia");
            EntryReal.readProperties(prop, Ic, "GlauertRotor.Blade_lagging_moment_of_inertia");
        } else {
            rb1.setSelected(false);
            rb2.setSelected(true);
            display("Computed");
            EntryReal.readProperties(prop, ml, "GlauertRotor.Blade_mass_per_unit_length");
        }
    }

    /**
     *
     * @param prop
     */
    public void write(Properties prop) {
        airfoilGUI.write(prop);
        inducedVelocityGUI.write(prop);
        EntryReal.writeProperties(prop, b, "GlauertRotor.Number_of_blades");
        EntryReal.writeProperties(prop, R, "GlauertRotor.Blade_length");
        EntryReal.writeProperties(prop, c, "GlauertRotor.Blade_chord");
        EntryReal.writeProperties(prop, xcg, "GlauertRotor.Chordwise_center_of_gravity_location");
        EntryReal.writeProperties(prop, thetaTW, "GlauertRotor.Blade_twist");
        EntryReal.writeProperties(prop, GJ, "GlauertRotor.Torsional_Rigidity");
        EntryReal.writeProperties(prop, B, "GlauertRotor.Tip_loss_factor");
        if (rb1.isSelected()) {
            prop.setProperty("GlauertRotor.Blade_moment_of_inertia_type", "Entered_by_user");
            EntryReal.writeProperties(prop, Ib, "GlauertRotor.Blade_flapping_moment_of_inertia");
            EntryReal.writeProperties(prop, Ic, "GlauertRotor.Blade_lagging_moment_of_inertia");
        } else {
            prop.setProperty("GlauertRotor.Blade_moment_of_inertia_type", "Computed");
            EntryReal.writeProperties(prop, ml, "GlauertRotor.Blade_mass_per_unit_length");
        }
    }

    /**
     *
     * @return
     */
    public GlauertRotor getGlauertRotor() {
        int bValue = Integer.parseInt(b.getText());
        if (bValue <= 0) {
            throw new IllegalArgumentException("Number of blade must be strictly positive");
        }
        double RValue = EntryReal.read(R, Units.UnitsList.length1);
        if (RValue <= 0) {
            throw new IllegalArgumentException("Blade length must be strictly positive");
        }
        double cValue = EntryReal.read(c, Units.UnitsList.length2);
        if (cValue <= 0) {
            throw new IllegalArgumentException("Blade chord must be strictly positive");
        }
        double BValue = EntryReal.read(B);
        if (BValue <= 0 || BValue > 1) {
            throw new IllegalArgumentException("Tip loss factor must be strictly positive and lower than unity");
        }
        double GJValue = EntryReal.read(GJ, Units.UnitsList.torsionalRigidity);
        if (GJValue < 0) {
            throw new IllegalArgumentException("Torsional rigidity must be positive");
        }

        double IbValue = EntryReal.read(Ib, Units.UnitsList.momentOfInertia);
        if (IbValue <= 0) {
            throw new IllegalArgumentException("Blade flapping moment of inertia must be strictly positive");
        }

        double IcValue = EntryReal.read(Ic, Units.UnitsList.momentOfInertia);
        if (IbValue <= 0) {
            throw new IllegalArgumentException("Blade lagging moment of inertia must be strictly positive");
        }
        if (rb1.isSelected()) {
            return new GlauertRotor(bValue, RValue, cValue,
                    Double.parseDouble(xcg.getText()),
                    EntryReal.read(thetaTW, Units.UnitsList.angle),
                    GJValue, BValue, IbValue, IcValue,
                    airfoilGUI.getAirfoilProfile(),
                    inducedVelocityGUI.getInducedVelocity());
        } else {
            RValue = Double.parseDouble(R.getText());
            final double mlValue = Double.parseDouble(ml.getText());
            final double I = mlValue * FastMath.pow(RValue, 3) / 3;
            return new GlauertRotor(bValue, RValue, cValue,
                    Double.parseDouble(xcg.getText()),
                    EntryReal.read(thetaTW, Units.UnitsList.angle),
                    GJValue, BValue, I, I,
                    airfoilGUI.getAirfoilProfile(),
                    inducedVelocityGUI.getInducedVelocity());

        }
    }
}
