/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
/*
03/10/15 : Computation of Gamma was erroneous.
*/
package gui.Rotor;

import gui.Utils.EntryReal;
import gui.Utils.UnitDependant;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JLabel;
import javax.swing.JPanel;
import main.rotor.GlauertRotor;
import main.utils.Units;
import net.miginfocom.swing.MigLayout;
import org.apache.commons.math3.util.FastMath;

/**
 *
 * @author J. Fourcade
 */
public class GlauertRotorResultsGUI implements UnitDependant {

    private GlauertRotor rotor = null;
    // Main panel
    private final JLabel gamma;
    private final JLabel s;
    private final JLabel Ma, MaUnit;
    private final JLabel P, PUnit;
    private final JLabel Gamma, GammaUnit;
    private final JLabel CL;
    private final JLabel CD;
    private final JLabel vi, viUnit;
    private final JLabel Vz1, Vz1Unit;
    private final JLabel eta;
    // Flapping panel
    private final JLabel mu;
    private final JLabel lambda, lambdaD;
    private final JLabel a0, a0Unit;
    private final JLabel a1, a1Unit;
    private final JLabel b1, b1Unit;
    private final JLabel a2, a2Unit;
    private final JLabel b2, b2Unit;
    private final JLabel nu, nuUnit;
    // Force and moment panel
    private final JLabel T, TUnit;
    private final JLabel Hp, HpUnit;
    private final JLabel Hi, HiUnit;
    private final JLabel H, HUnit, HD, HDUnit;
    private final JLabel Qp, QpUnit;
    private final JLabel Qi, QiUnit;
    private final JLabel Q, QUnit;
    private final JLabel L, LUnit;
    private final JLabel D, DUnit;
    private final JLabel Yi, YiUnit, YiD, YiDUnit;
    // Panels
    private final JPanel mainPanel, coefficientsPanel, forceAndTorquePanel;

    class RadioListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            if (rotor != null) {
                setValues(rotor);
            }
        }
    }

    /**
     *
     */
    public GlauertRotorResultsGUI() {

        // Panels
        coefficientsPanel = new JPanel(new MigLayout("insets 5 5 5 5,fillx", "[]15[]15[]5[]"));
        mainPanel = new JPanel(new MigLayout("insets 5 5 5 5,fillx", "[]15[]15[]5[]"));
        forceAndTorquePanel = new JPanel(new MigLayout("insets 5 5 5 5", "[]15[]15[]5[]"));

        // Panel font and title font
        Font panelFont = new Font("Courier", Font.PLAIN, 12);
        coefficientsPanel.setFont(panelFont);
        mainPanel.setFont(panelFont);
        forceAndTorquePanel.setFont(panelFont);
        Font font = new Font("Courier", Font.BOLD, 16);

        // Inflow ratio and flapping coefficients panel ------------------------
        // Title
        JLabel coefsTitle = new JLabel("Flapping coefficients :");
        coefsTitle.setFont(font);
        coefficientsPanel.add(coefsTitle, "spanx 3, wrap");

        // Advance ratio
        coefficientsPanel.add(new JLabel("Advance ratio"));
        coefficientsPanel.add(new JLabel(" μ"));
        mu = new JLabel("0.0000");
        mu.setFont(font);
        coefficientsPanel.add(mu, "wrap");

        // Inflow ratio
        coefficientsPanel.add(new JLabel("Inflow ratio"));
        coefficientsPanel.add(new JLabel(" λ"));
        lambda = new JLabel("0.0000");
        lambda.setFont(font);
        coefficientsPanel.add(lambda, "wrap");

        // Inflow ratio (TPP)
        coefficientsPanel.add(new JLabel("Inflow ratio TPP"));
        coefficientsPanel.add(new JLabel(" λD"));
        lambdaD = new JLabel("0.0000");
        lambdaD.setFont(font);
        coefficientsPanel.add(lambdaD, "wrap");

        // Conning angle
        coefficientsPanel.add(new JLabel("Conning angle"));
        coefficientsPanel.add(new JLabel(" a0"));
        a0 = new JLabel("0.00");
        a0.setFont(font);
        coefficientsPanel.add(a0);
        a0Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        coefficientsPanel.add(a0Unit, "wrap");

        // Longitudinal flapping
        coefficientsPanel.add(new JLabel("Longitudinal flapping"));
        coefficientsPanel.add(new JLabel(" a1"));
        a1 = new JLabel("0.00");
        a1.setFont(font);
        coefficientsPanel.add(a1);
        a1Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        coefficientsPanel.add(a1Unit, "wrap");

        // Latteral flapping
        coefficientsPanel.add(new JLabel("Lateral flapping"));
        coefficientsPanel.add(new JLabel(" b1"));
        b1 = new JLabel("0.00");
        b1.setFont(font);
        coefficientsPanel.add(b1);
        b1Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        coefficientsPanel.add(b1Unit, "wrap");

        // 2nd Longitudinal flapping
        coefficientsPanel.add(new JLabel("2nd longitudinal flapping"));
        coefficientsPanel.add(new JLabel(" a2"));
        a2 = new JLabel("0.000");
        a2.setFont(font);
        coefficientsPanel.add(a2);
        a2Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        coefficientsPanel.add(a2Unit, "wrap");

        // 2nd Lateral flapping
        coefficientsPanel.add(new JLabel("2nd lateral flapping"));
        coefficientsPanel.add(new JLabel(" b2"));
        b2 = new JLabel("0.000");
        b2.setFont(font);
        coefficientsPanel.add(b2);
        b2Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        coefficientsPanel.add(b2Unit, "wrap");

        // Torsionnal deformation
        coefficientsPanel.add(new JLabel("3/4 radius torsional deformation"));
        coefficientsPanel.add(new JLabel(" u0"));
        nu = new JLabel("0.000");
        nu.setFont(font);
        coefficientsPanel.add(nu);
        nuUnit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        coefficientsPanel.add(nuUnit, "wrap");

        // Constants panel -----------------------------------------------------
        // Title
        JLabel mainTitle = new JLabel("Main results :");
        mainTitle.setFont(font);
        mainPanel.add(mainTitle, "spanx 2, wrap");

        // Lock Number
        mainPanel.add(new JLabel("Lock Number"));
        mainPanel.add(new JLabel(" γ"));
        gamma = new JLabel("0.00");
        gamma.setFont(font);
        mainPanel.add(gamma, "wrap");

        // Solidity
        mainPanel.add(new JLabel("Solidity"));
        mainPanel.add(new JLabel(" s"));
        s = new JLabel("0.000");
        s.setFont(font);
        mainPanel.add(s, "wrap");

        // Blade Flapping Moment
        mainPanel.add(new JLabel("Blade flapping moment"));
        mainPanel.add(new JLabel(" M"));
        Ma = new JLabel("0.00");
        Ma.setFont(font);
        mainPanel.add(Ma);
        MaUnit = new JLabel(Units.getLabel(Units.UnitsList.moment));
        mainPanel.add(MaUnit, "wrap");

        // Power to drive rotor
        mainPanel.add(new JLabel("Power to drive rotor"));
        mainPanel.add(new JLabel(" P"));
        P = new JLabel(Double.toString(0.0));
        P.setFont(font);
        mainPanel.add(P);
        PUnit = new JLabel(Units.getLabel(Units.UnitsList.power));
        mainPanel.add(PUnit, "wrap");

        // Rotor acceleration
        mainPanel.add(new JLabel("Rotor acceleration"));
        mainPanel.add(new JLabel(" Γ"));
        Gamma = new JLabel("0.00");
        Gamma.setFont(font);
        mainPanel.add(Gamma);
        GammaUnit = new JLabel(Units.getLabel(Units.UnitsList.rpmAcceleration));
        mainPanel.add(GammaUnit, "wrap");

        // Rotor CL
        mainPanel.add(new JLabel("Rotor lift coefficient"));
        mainPanel.add(new JLabel(" CL"));
        CL = new JLabel("0.000");
        CL.setFont(font);
        mainPanel.add(CL, "wrap");

        // Rotor CD
        mainPanel.add(new JLabel("Rotor drag coefficient"));
        CD = new JLabel("0.000");
        mainPanel.add(new JLabel(" CD"));
        CD.setFont(font);
        mainPanel.add(CD, "wrap");

        // Induced velocity
        mainPanel.add(new JLabel("Induced velocity"));
        mainPanel.add(new JLabel(" vi"));
        vi = new JLabel("0.00");
        vi.setFont(font);
        mainPanel.add(vi);
        viUnit = new JLabel(Units.getLabel(Units.UnitsList.velocity2));
        mainPanel.add(viUnit, "wrap");

        // Axial velocity
        mainPanel.add(new JLabel("Axial velocity"));
        mainPanel.add(new JLabel(" Vz1"));
        Vz1 = new JLabel("0.00");
        Vz1.setFont(font);
        mainPanel.add(Vz1);
        Vz1Unit = new JLabel(Units.getLabel(Units.UnitsList.velocity2));
        mainPanel.add(Vz1Unit, "wrap");

        // Working state
        mainPanel.add(new JLabel("Working state"));
        mainPanel.add(new JLabel(" η"));
        eta = new JLabel("0.00");
        eta.setFont(font);
        mainPanel.add(eta, "wrap");

        // Force and moment panel ----------------------------------------------
        // Title
        JLabel forceAndTorqueTitle = new JLabel("Forces and torque :");
        forceAndTorqueTitle.setFont(font);
        forceAndTorquePanel.add(forceAndTorqueTitle, "spanx 2, wrap");

        // Rotor thrust
        forceAndTorquePanel.add(new JLabel("Rotor thrust"));
        forceAndTorquePanel.add(new JLabel(" T"));
        T = new JLabel("0.00");
        T.setFont(font);
        forceAndTorquePanel.add(T);
        TUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        forceAndTorquePanel.add(TUnit, "wrap");

        // Induced lateral force
        forceAndTorquePanel.add(new JLabel("Lateral force"));
        forceAndTorquePanel.add(new JLabel(" Y"));
        Yi = new JLabel("0.00");
        Yi.setFont(font);
        forceAndTorquePanel.add(Yi);
        YiUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        forceAndTorquePanel.add(YiUnit, "wrap");

        // Induced lateral force (TPP)
        forceAndTorquePanel.add(new JLabel("Lateral force TPP"));
        forceAndTorquePanel.add(new JLabel(" YD"));
        YiD = new JLabel("0.00");
        YiD.setFont(font);
        forceAndTorquePanel.add(YiD);
        YiDUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        forceAndTorquePanel.add(YiDUnit, "wrap");

        // Rear profile drag force
        forceAndTorquePanel.add(new JLabel("Rear profil drag force"));
        forceAndTorquePanel.add(new JLabel(" Hp"));
        Hp = new JLabel("0.00");
        Hp.setFont(font);
        forceAndTorquePanel.add(Hp);
        HpUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        forceAndTorquePanel.add(HpUnit, "wrap");

        // Rear induced force 
        forceAndTorquePanel.add(new JLabel("Rear induced force"));
        forceAndTorquePanel.add(new JLabel(" Hi"));
        Hi = new JLabel("0.00");
        Hi.setFont(font);
        forceAndTorquePanel.add(Hi);
        HiUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        forceAndTorquePanel.add(HiUnit, "wrap");

        // Rear total force
        forceAndTorquePanel.add(new JLabel("Rear total force"));
        forceAndTorquePanel.add(new JLabel(" H"));
        H = new JLabel("0.00");
        H.setFont(font);
        forceAndTorquePanel.add(H);
        HUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        forceAndTorquePanel.add(HUnit, "wrap");

        // Rear total force (TPP)
        forceAndTorquePanel.add(new JLabel("Rear total force TPP"));
        forceAndTorquePanel.add(new JLabel(" HD"));
        HD = new JLabel("0.00");
        HD.setFont(font);
        forceAndTorquePanel.add(HD);
        HDUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        forceAndTorquePanel.add(HDUnit, "wrap");

        // Profil drag moment
        forceAndTorquePanel.add(new JLabel("Profil drag torque"));
        forceAndTorquePanel.add(new JLabel(" Qp"));
        Qp = new JLabel("0.00");
        Qp.setFont(font);
        forceAndTorquePanel.add(Qp);
        QpUnit = new JLabel(Units.getLabel(Units.UnitsList.moment));
        forceAndTorquePanel.add(QpUnit, "wrap");

        // Induced moment
        forceAndTorquePanel.add(new JLabel("Induced torque"));
        forceAndTorquePanel.add(new JLabel(" Qi"));
        Qi = new JLabel("0.00");
        Qi.setFont(font);
        forceAndTorquePanel.add(Qi);
        QiUnit = new JLabel(Units.getLabel(Units.UnitsList.moment));
        forceAndTorquePanel.add(QiUnit, "wrap");

        // Total moment
        forceAndTorquePanel.add(new JLabel("Total torque"));
        forceAndTorquePanel.add(new JLabel(" Q"));
        Q = new JLabel("0.00");
        Q.setFont(font);
        forceAndTorquePanel.add(Q);
        QUnit = new JLabel(Units.getLabel(Units.UnitsList.moment));
        forceAndTorquePanel.add(QUnit, "wrap");

        // Rotor lift
        forceAndTorquePanel.add(new JLabel("Rotor lift"));
        forceAndTorquePanel.add(new JLabel(" L"));
        L = new JLabel("0.00");
        L.setFont(font);
        forceAndTorquePanel.add(L);
        LUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        forceAndTorquePanel.add(LUnit, "wrap");

        // Rotor drag
        forceAndTorquePanel.add(new JLabel("Rotor drag"));
        forceAndTorquePanel.add(new JLabel(" D"));
        D = new JLabel("0.00");
        D.setFont(font);
        forceAndTorquePanel.add(D);
        DUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        forceAndTorquePanel.add(DUnit, "wrap");
    }

    /**
     *
     * @param rotor
     */
    public void setValues(GlauertRotor rotor) {

        this.rotor = rotor;
        double tmp;

        // Main Panel
        EntryReal.set(gamma, rotor.getLockNumber(), "%.2f");
        EntryReal.set(s, rotor.getSolidity(), "%.3f");
        tmp = rotor.getConingAngle() * rotor.getBladeFlappingMomentOfInertia()
                * FastMath.pow(rotor.getRotorSpeed(), 2);
        EntryReal.set(Ma, tmp, "%.2f", Units.UnitsList.moment);
        tmp = -rotor.getTotalTorque() * rotor.getRotorSpeed();
        EntryReal.set(P, tmp, "%.2f", Units.UnitsList.power);
        tmp = rotor.getTotalTorque() / rotor.getNumberOfBlade()/
                rotor.getBladeLaggingMomentOfInertia();
        EntryReal.set(Gamma, tmp, "%.2f", Units.UnitsList.rpmAcceleration);
        EntryReal.set(CL, rotor.getLiftCoefficient(), "%.3f");
        EntryReal.set(CD, rotor.getDragCoefficient(), "%.3f");
        EntryReal.set(vi, rotor.getInducedVelocity().getVelocity(), "%.2f",
                Units.UnitsList.velocity2);
        tmp = rotor.getVelocity() * FastMath.sin(rotor.getIncidence())
                - rotor.getInducedVelocity().getVelocity();
        EntryReal.set(Vz1, tmp, "%.2f", Units.UnitsList.velocity2);
        tmp = -rotor.getVelocity() * FastMath.sin(rotor.getIncidence())
                / rotor.getInducedVelocity().getVelocity();
        EntryReal.set(eta, tmp, "%.2f");

        // Flapping panel
        EntryReal.set(mu, rotor.getAdvanceRatio(), "%.4f");
        EntryReal.set(lambda, rotor.getInFlowRatio(), "%.4f");
        final double lambdaTPP = rotor.getInFlowRatio() + rotor.getAdvanceRatio()
                * rotor.getLongitudinalFlapping();
        EntryReal.set(lambdaD, lambdaTPP, "%.4f");
        EntryReal.set(a0, rotor.getConingAngle(), "%.2f",
                Units.UnitsList.angle);
        EntryReal.set(a1, rotor.getLongitudinalFlapping(), "%.2f",
                Units.UnitsList.angle);
        EntryReal.set(b1, rotor.getLateralFlapping(), "%.2f",
                Units.UnitsList.angle);
        EntryReal.set(a2, rotor.get2ndLongitudinalFlapping(), "%.3f",
                Units.UnitsList.angle);
        EntryReal.set(b2, rotor.get2ndLateralFlapping(), "%.3f",
                Units.UnitsList.angle);
        EntryReal.set(nu, rotor.computeMeanPitchDeformation(0.75), "%.3f",
                Units.UnitsList.angle);

        // Force and moment panel
        EntryReal.set(T, rotor.getThrust(), "%.2f",
                Units.UnitsList.force);
        EntryReal.set(Yi, rotor.getLateralForce(), "%.2f", Units.UnitsList.force);
        final double YiTPP = rotor.getLateralForce() - rotor.getLateralFlapping()
                * rotor.getThrust();
        EntryReal.set(YiD, YiTPP, "%.2f", Units.UnitsList.force);
        EntryReal.set(Hp, rotor.getProfileDragRearForce(), "%.2f", Units.UnitsList.force);
        EntryReal.set(Hi, rotor.getInducedRearForce(), "%.2f", Units.UnitsList.force);
        EntryReal.set(H, rotor.getTotalRearForce(), "%.2f", Units.UnitsList.force);
        final double HTPP = rotor.getTotalRearForce() - rotor.getLongitudinalFlapping()
                * rotor.getThrust();
        EntryReal.set(HD, HTPP, "%.2f", Units.UnitsList.force);
        EntryReal.set(Qp, rotor.getProfileDragTorque(), "%.2f",
                Units.UnitsList.moment);
        EntryReal.set(Qi, rotor.getInducedTorque(), "%.2f",
                Units.UnitsList.moment);
        EntryReal.set(Q, rotor.getTotalTorque(), "%.2f",
                Units.UnitsList.moment);
        EntryReal.set(L, rotor.getLift(), "%.2f", Units.UnitsList.force);
        EntryReal.set(D, rotor.getDrag(), "%.2f", Units.UnitsList.force);
    }

    /**
     *
     */
    @Override
    public void unitsUpdate() {

        // Main panel
        EntryReal.unitUpdate(Ma, "%.2f", MaUnit, Units.UnitsList.moment);
        EntryReal.unitUpdate(P, "%.2f", PUnit, Units.UnitsList.power);

        // Flapping panel
        EntryReal.unitUpdate(vi, "%.2f", viUnit, Units.UnitsList.velocity2);
        EntryReal.unitUpdate(Vz1, "%.2f", Vz1Unit, Units.UnitsList.velocity2);

        // Force and moment panel
        EntryReal.unitUpdate(T, "%.2f", TUnit, Units.UnitsList.force);
        EntryReal.unitUpdate(Yi, "%.2f", YiUnit, Units.UnitsList.force);
        EntryReal.unitUpdate(YiD, "%.2f", YiDUnit, Units.UnitsList.force);
        EntryReal.unitUpdate(Hp, "%.2f", HpUnit, Units.UnitsList.force);
        EntryReal.unitUpdate(Hi, "%.2f", HiUnit, Units.UnitsList.force);
        EntryReal.unitUpdate(H, "%.2f", HUnit, Units.UnitsList.force);
        EntryReal.unitUpdate(Qp, "%.2f", QpUnit, Units.UnitsList.moment);
        EntryReal.unitUpdate(Qi, "%.2f", QiUnit, Units.UnitsList.moment);
        EntryReal.unitUpdate(Q, "%.2f", QUnit, Units.UnitsList.moment);
        EntryReal.unitUpdate(L, "%.2f", LUnit, Units.UnitsList.force);
        EntryReal.unitUpdate(D, "%.2f", DUnit, Units.UnitsList.force);
    }

    /**
     *
     * @return
     */
    public JPanel getMainPanel() {
        return mainPanel;
    }

    /**
     *
     * @return
     */
    public JPanel getCoefficientPanel() {
        return coefficientsPanel;
    }

    /**
     *
     * @return
     */
    public JPanel getForceAndTorquePanel() {
        return forceAndTorquePanel;
    }
}
