/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gui.Rotor;

import gui.Utils.UnitDependant;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import main.rotor.GlauertRotor;
import net.miginfocom.swing.MigLayout;
import org.apache.commons.math3.geometry.euclidean.threed.Vector3D;

/**
 *
 * @author J. Fourcade
 */
public class RotorDerivativesGUI extends JPanel implements UnitDependant {

    // Display WR disk or fuselage
    private final JRadioButton rb1, rb2;

    // Panels
    private final RotorDerivativesWRTDiscGUI derivativesDiscPanel;
    private final RotorDerivativesWRTFuselageGUI derivativesFuselagePanel;

    class RadioListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            if (e.getActionCommand().equals("disc")) {
                derivativesDiscPanel.setVisible(true);
                derivativesFuselagePanel.setVisible(false);
            } else {
                derivativesDiscPanel.setVisible(false);
                derivativesFuselagePanel.setVisible(true);
            }
        }
    }

    /**
     *
     */
    public RotorDerivativesGUI() {

        // Set layout
        setLayout(new MigLayout("hidemode 3,insets 5 5 5 5"));

        // Panels
        derivativesDiscPanel = new RotorDerivativesWRTDiscGUI();
        derivativesFuselagePanel = new RotorDerivativesWRTFuselageGUI();

        // Panel font and title font
        Font panelFont = new Font("Courier", Font.PLAIN, 12);
        setFont(panelFont);

        // Non-dimensionalisation mode
        rb1 = new JRadioButton("With respect to no-feathering coordinate system");
        rb1.setActionCommand("disc");
        rb1.setSelected(true);
        rb2 = new JRadioButton("With respect to fuselage coordinate system");
        rb2.setActionCommand("fuselage");
        add(rb1, "wrap");
        add(rb2, "wrap 20px");

        // Group
        ButtonGroup group = new ButtonGroup();
        group.add(rb1);
        group.add(rb2);

        // Listener
        RadioListener myListener = new RadioListener();
        rb1.addActionListener(myListener);
        rb2.addActionListener(myListener);

        // Derivatives with respect to rotor disk parameters
        add(derivativesDiscPanel, "spanx 10");

        // Derivatives with respect to fuselage parameters
        add(derivativesFuselagePanel, "spanx 10");
        derivativesFuselagePanel.setVisible(false);

    }

    public double getFuselageIncidence() {
        return derivativesFuselagePanel.getFuselageIncidence();
    }

    public Vector3D getHubPosition() {
        return derivativesFuselagePanel.getHubPosition();
    }

    /**
     *
     * @param rotor
     */
    public void setValues(GlauertRotor rotor) {

        // Write derivatives
        derivativesFuselagePanel.setValues(rotor);
        derivativesDiscPanel.setValues(rotor);
    }

    @Override
    public void unitsUpdate() {

        derivativesFuselagePanel.unitsUpdate();
        derivativesDiscPanel.unitsUpdate();
    }
}
