/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and, more generally,to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gui.Utils;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Properties;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.SwingUtilities;
import main.utils.Units;

/**
 *
 * @author J. Fourcade
 */
public abstract class AboutMenu extends JMenu {

    private final JFrame frame;
    private final JMenuItem about, quit;
    private final JRadioButtonMenuItem rb1, rb2;
    private final String name, version, function;

    class MyItemsListener implements ActionListener, ItemListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            if (e.getSource() == about) {
                final JPanel aboutPanel = new JPanel();
                aboutPanel.setLayout(new BoxLayout(aboutPanel, BoxLayout.Y_AXIS));
                final String t1 = "<html>" + name + " Version "+version
                        + "<p>Copyright J. Fourcade."
                        + "<p>E-mail : <a href=\"mailto:autogire@volucres.fr\">autogire@volucres.fr</a>"
                        + "<p>Web site : <a href=\"http://www.volucres.fr\">http://www.volucres.fr</a></html>";
                final String t2 = "<html>"
                        + "<p><p>This software is a computer program whose purpose is to compute "
                        + "<p>"+function
                        + "<p><p>This software is governed by the CeCILL license under French law and"
                        + "<p>abiding by the rules of distribution of free software.  You can  use,"
                        + "<p>modify and/ or redistribute the software under the terms of the CeCILL"
                        + "<p>license as circulated by CEA, CNRS and INRIA at the following URL"
                        + "<p><a href=\"http://www.cecill.info\">http://www.cecill.info</a>.</html>";
                final String t3 = "<html>"
                        + "<p><p>As a counterpart to the access to the source code and  rights to copy,"
                        + "<p>modify and redistribute granted by the license, users are provided only"
                        + "<p>with a limited warranty  and the software's author,  the holder of the"
                        + "<p>economic rights,  and the successive licensors  have only  limited"
                        + "<p>liability.</html>";
                final String t4 = "<html>"
                        + "<p><p>In this respect, the user's attention is drawn to the risks associated"
                        + "<p>with loading,  using,  modifying and/or developing or reproducing the"
                        + "<p>software by the user in light of its specific status of free software,"
                        + "<p>that may mean  that it is complicated to manipulate,  and  that  also"
                        + "<p>therefore means  that it is reserved for developers  and  experienced"
                        + "<p>professionals having in-depth computer and aerodynamic knowledge. "
                        + "<p>Users are therefore encouraged to load and test the software's suitability"
                        + "<p>as regards their requirements in conditions enabling the security of their"
                        + "<p>systems and/or data to be ensured and,  more generally, to use and"
                        + "<p>operate it in the same conditions as regards security.</html>";
                final String t5 = "<html>"
                        + "<p><p>The fact that you are presently reading this means that you have had"
                        + "<p>knowledge of the CeCILL license and that you accept its terms.</html>";

                aboutPanel.add(new JLabel(t1));
                aboutPanel.add(new JLabel(t2));
                aboutPanel.add(new JLabel(t3));
                aboutPanel.add(new JLabel(t4));
                aboutPanel.add(new JLabel(t5));
                JOptionPane.showMessageDialog(frame, aboutPanel);
            }
            if (e.getSource() == rb1) {
                Units.mode = Units.Mode.metric;
                unitsUpdate();
            }
            if (e.getSource() == rb2) {
                Units.mode = Units.Mode.british;
                unitsUpdate();
            }
            if (e.getSource() == quit) {
                System.exit(0);
            }
        }

        @Override
        public void itemStateChanged(ItemEvent e) {
        }
    }

    /**
     *
     * @param name
     * @param version
     * @param function
     */
    public AboutMenu(String name, String version, String function) {
        super(name);
        this.name = name;
        this.version = version;
        this.function = function;

        MyItemsListener listener = new MyItemsListener();

        about = new JMenuItem("About " + name);
        add(about);
        about.addActionListener(listener);
        addSeparator();

        ButtonGroup group = new ButtonGroup();
        rb1 = new JRadioButtonMenuItem("Metric units");
        rb1.setActionCommand("Metric");
        rb1.setSelected(true);
        group.add(rb1);
        add(rb1);
        rb1.addActionListener(listener);

        rb2 = new JRadioButtonMenuItem("British units");
        rb2.setActionCommand("British");
        group.add(rb2);
        add(rb2);
        rb2.addActionListener(listener);
        addSeparator();

        quit = new JMenuItem("Quit " + name);
        add(quit);
        quit.addActionListener(listener);

        // Get frame for dialog
        Component component = (Component) this;
        frame = (JFrame) SwingUtilities.getRoot(component);
    }

    /**
     *
     * @param prop
     */
    public void writeUnitsProperties(Properties prop) {
        prop.setProperty("Units", Units.getMode());
    }

    /**
     *
     * @param prop
     */
    public void readUnitsProperties(Properties prop) {
        String unit = prop.getProperty("Units");
        if (unit.equals("Metric") && Units.mode != Units.Mode.metric) {
            Units.mode = Units.Mode.metric;
            rb1.setSelected(true);
            unitsUpdate();
        }
        if (unit.equals("British") && Units.mode != Units.Mode.british) {
            Units.mode = Units.Mode.british;
            rb2.setSelected(true);
            unitsUpdate();
        }
    }

    /**
     *
     */
    public abstract void unitsUpdate();
}