/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and, more generally,to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package main.atmosphere;

/**
 * Density atmosphere use only density parameter. Others parameters (temperature, 
 * pressure, brake power altitude factor, speed of sound, acceleration of gravity
 * and kinematic viscosity are unavailable).
 *
 * @author J. Fourcade
 */
public class DensityAtmosphere extends Atmosphere {
    
    final double density;
    
    /**
     * Constructor defining  density.
     *
     * @param density : air density (kg/m3)
     */
    public DensityAtmosphere(double density) {
        this.density = density;
    }
    
    
    @Override
    public double getDensity() {
        return density; 
    }

    @Override
    public double getTemperature() {
        throw new UnsupportedOperationException("DensityAtmosphere does not implement getTemperature"); 
    }

    @Override
    public double getPressure() {
        throw new UnsupportedOperationException("DensityAtmosphere does not implement getPressure"); 
    }

    @Override
    public double getBrakePowerAltitudeFactor() {
        throw new UnsupportedOperationException("DensityAtmosphere does not implement getBrakePowerAltitudeFactor"); 
    }

    @Override
    public double getSpeedOfSound() {
        throw new UnsupportedOperationException("DensityAtmosphere does not implement getSpeedOfSound"); 
    }

    @Override
    public double getAccelerationOfGravity() {
        throw new UnsupportedOperationException("DensityAtmosphere does not implement getAccelerationOfGravity"); 
    }

    @Override
    public double getKinematicViscosity() {
        throw new UnsupportedOperationException("DensityAtmosphere does not implement getKinematicViscosity"); 
    }

    @Override
    public String getName() {
        return "Only Density Atmosphere";
    }
    
}
