/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gyrorotor;

import gui.Utils.EntryReal;
import gui.Utils.UnitDependant;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import main.utils.Units;
import net.miginfocom.swing.MigLayout;

/**
 * Flight data panel
 * 
 * @author J. Fourcade
 */
public class FlightData extends JPanel implements UnitDependant {

    // Datas
    JRadioButton rb1, rb2;
    JTextField U, alphaS, theta0, q, p, omega;
    JLabel UUnit, alphaSUnit, theta0Unit, qUnit, pUnit, omegaUnit;
    // Font
    Font font;

    class RadioListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            if (e.getActionCommand().equals("Autorotative")) {
                omega.setEditable(false);
            } else {
                omega.setEditable(true);
            }
        }
    }

    /**
     * Constructor
     * 
     */
    public FlightData() {

        // Set layout
        setLayout(new MigLayout("insets 5 5 5 5", "[]15[]15[]5[]"));

        // Panel font and title font
        setFont(new Font("Courier", Font.PLAIN, 12));
        font = getFont().deriveFont(Font.BOLD, 16);

        // Title
        JLabel title = new JLabel("Flight datas :");
        title.setFont(font);
        this.add(title, "spanx 2, wrap");

        // Mode (helicopter or autogyro)
        rb1 = new JRadioButton("Autorotative mode");
        rb1.setActionCommand("Autorotative");
        rb2 = new JRadioButton("Fixed rotor speed");
        rb2.setActionCommand("Fixed");
        rb1.setSelected(true);
        add(rb1);
        add(rb2, "spanx 3, wrap");

        // Group
        ButtonGroup group = new ButtonGroup();
        group.add(rb1);
        group.add(rb2);

        // Listener
        RadioListener myListener = new RadioListener();
        rb1.addActionListener(myListener);
        rb2.addActionListener(myListener);

        // Flight velocity
        add(new JLabel("Flight Velocity"));
        add(new JLabel(" U"));
        U = new JTextField("0.0000");
        add(U, "w 80");
        UUnit = new JLabel(Units.getLabel(Units.UnitsList.velocity1));
        add(UUnit, "wrap");

        // Rotor disk incidence (no-feathering)
        add(new JLabel("Incidence NFP"));
        add(new JLabel(" αS"));
        alphaS = new JTextField("0.0");
        add(alphaS, "w 80");
        alphaSUnit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        add(alphaSUnit, "wrap");

        // Rotor disk incidence (no-feathering)
        add(new JLabel("Blade Pitch"));
        add(new JLabel(" θ0"));
        theta0 = new JTextField("0.0");
        add(theta0, "w 80");
        theta0Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        add(theta0Unit, "wrap");

        // Pitch rate
        add(new JLabel("Pitch rate"));
        add(new JLabel(" q"));
        q = new JTextField("0.0");
        add(q, "w 80");
        qUnit = new JLabel(Units.getLabel(Units.UnitsList.angularVelocity));
        add(qUnit, "wrap");

        // Roll rate
        add(new JLabel("Roll rate"));
        add(new JLabel(" p"));
        p = new JTextField("0.0");
        add(p, "w 80");
        pUnit = new JLabel(Units.getLabel(Units.UnitsList.angularVelocity));
        add(pUnit, "wrap");

        // Rotor speed
        add(new JLabel("Rotor speed"));
        add(new JLabel(" Ω"));
        omega = new JTextField("0.0");
        omega.setEditable(false);
        add(omega, "w 80");
        omegaUnit = new JLabel(Units.getLabel(Units.UnitsList.rpm));
        add(omegaUnit, "wrap");
    }

    /**
     *
     * @return
     */
    public double getFlightVelocity() {
        final double UValue = EntryReal.read(U, Units.UnitsList.velocity1);
        if (UValue < 0) {
            throw new IllegalArgumentException("Velocity must be positive");
        }
        return UValue;
    }

    /**
     * Get rotor no-feathering plane incidence
     * 
     * @return incidence (rad)
     */
    public double getIncidence() {
        return EntryReal.read(alphaS, Units.UnitsList.angle);
    }

    /**
     * Get blade pitch at root entered by user
     * 
     * @return blade pitch (rad)
     */
    public double getBladePitch() {
        return EntryReal.read(theta0, Units.UnitsList.angle);
    }

    /**
     * Get pitch rate entered by user
     * 
     * @return pitch rate (rad/s)
     */
    public double getPitchRate() {
        return EntryReal.read(q, Units.UnitsList.angularVelocity);
    }

    /**
     * Get roll rate entered bu user
     * 
     * @return roll rate (rad/s)
     */
    public double getRollRate() {
        return EntryReal.read(p, Units.UnitsList.angularVelocity);
    }

    /**
     * Get rotor speed entered by user
     * 
     * @return rotor speed (rad/s)
     */
    public double getRotorSpeed() {
        return EntryReal.read(omega, Units.UnitsList.rpm);
    }

    /**
     * Get rotor mode (true if rotor is in autorotative state)
     * 
     * @return rotor mode
     */
    public boolean isAutorotativeMode() {
        return rb1.isSelected();
    }

    /**
     * Set rotor speed
     * 
     * @param rotorSpeed (m/s)
     */
    public void setRotorSpeed(double rotorSpeed) {
        EntryReal.set(omega, rotorSpeed, "%.2f", Units.UnitsList.rpm);
    }

    /**
     * Unit updates
     */
    @Override
    public void unitsUpdate() {
        EntryReal.unitUpdate(U, "%.4f", UUnit, Units.UnitsList.velocity1);
    }
}
