/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
/*
03/10/15 : Suppression of negative incidence test when in auto rotative mode.
*/
package gyrorotor;

import gui.Rotor.AoaDistributionChart;
import gui.Atmosphere.AtmosphereGUI;
import gui.Rotor.RotorDerivativesGUI;
import gui.Rotor.GlauertRotorGUI;
import gui.Rotor.GlauertRotorResultsGUI;
import gui.Utils.AboutMenu;
import gui.Utils.FileNameGUI;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Locale;
import java.util.Properties;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.filechooser.FileNameExtensionFilter;
import main.airfoil.ConstantAirfoil;
import main.atmosphere.Atmosphere;
import main.atmosphere.DensityAtmosphere;
import main.inducedvelocity.ConstantInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import main.rotor.GlauertRotor;
import net.miginfocom.swing.MigLayout;

/**
 * Gyro rotor program
 * 
 * @author J. Fourcade
 */
public class GyroRotor extends JPanel {

    // Input parameters
    private Atmosphere atmosphere;
    private GlauertRotor rotor;
    private FlightData flightData;
    private final AoaDistributionData aoaDistributionData;
    // GUI
    final private AtmosphereGUI atmosphereGUI;
    final private GlauertRotorGUI glauertRotorGUI;
    final private GlauertRotorResultsGUI rotorResultsGUI;
    final private RotorDerivativesGUI rotorDerivativesGUI;
    // Actions
    JButton simulateButton, derivativesButton, aoaDistributionButton;
    // Menus
    final private JMenuBar menuBar;
    final private AboutMenu aboutMenu;
    //final private JMenuBar menuBar;
    final private JMenuItem openFile, saveFile, saveFileAs;
    // Polar Chart
    AoaDistributionChart aoaDistributionChart;
    // File name
    FileNameGUI fileNameGUI;
    private File fileName = null;
    private File directory = null;
    // Frame name
    static JFrame frame;

    class Simulate implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            // Compute rotor characteristics
            try {
                // Get data
                atmosphere = atmosphereGUI.getAtmosphere();
                rotor = glauertRotorGUI.getGlauertRotor();
                final double U = flightData.getFlightVelocity();
                final double alphaS = flightData.getIncidence();
                final double theta0 = flightData.getBladePitch();
                final double q = flightData.getPitchRate();
                final double p = flightData.getRollRate();
                final double omega;
                if (!flightData.isAutorotativeMode()
                        || e.getSource() == derivativesButton) {
                    omega = flightData.getRotorSpeed();
                    if (omega <= 0) {
                        throw new RuntimeException("Rotor speed must be strictly positive");
                    }
                } else {
                    omega = 0;
                }
                if (flightData.isAutorotativeMode()) {
                    // Autogyro mode
                    if (U == 0) {
                        throw new RuntimeException(
                                "Velocity must be strictly positive");
                    }
                }
                // Compute rotor results
                if (e.getSource() == simulateButton
                        || e.getSource() == aoaDistributionButton) {
                    try {
                        if (flightData.isAutorotativeMode()) {
                            rotor.computeState(U, alphaS, theta0, q, p,
                                    atmosphere);
                        } else {
                            rotor.computeState(U, alphaS, theta0, q, p, omega,
                                    atmosphere);
                        }
                        flightData.setRotorSpeed(rotor.getRotorSpeed());
                        rotorResultsGUI.setValues(rotor);

                    } catch (Exception ex) {
                        // Open error dialog 
                        JOptionPane.showMessageDialog(null, ex.getMessage(), "Error",
                                JOptionPane.ERROR_MESSAGE);
                    }
                }
                // Compute rotor derivatives
                if (e.getSource() == derivativesButton) {
                    rotor.computeDerivatives(U, alphaS, theta0, q, p, omega,
                            rotorDerivativesGUI.getFuselageIncidence(),
                            rotorDerivativesGUI.getHubPosition(), atmosphere);
                    rotorDerivativesGUI.setValues(rotor);
                }
                // Compute aoa distribution
                if (e.getSource() == aoaDistributionButton) {
                    aoaDistributionChart.computeAoaDistribution(rotor,
                            aoaDistributionData.getAoaValues(),
                            AoaDistributionData.getAoaColors());
                    aoaDistributionChart.repaint();
                }
            } catch (RuntimeException ex) {
                // Open error dialog 
                JOptionPane.showMessageDialog(null, ex.getMessage(), "Error",
                        JOptionPane.ERROR_MESSAGE);
            }
            frame.pack();
            frame.setMinimumSize(frame.getSize());
        }
    }

    class MyItemsListener implements ActionListener, ItemListener {

        @Override
        public void actionPerformed(ActionEvent e) {

            // Create a file chooser
            final JFileChooser fc = new JFileChooser();
            if (directory == null) {
                fc.setCurrentDirectory(new File(System.getProperty("user.dir")));
            } else {
                fc.setCurrentDirectory(directory);
            }
            FileNameExtensionFilter filter = new FileNameExtensionFilter(
                    "Rotor datas", "rot");
            fc.setFileFilter(filter);

            // File name and save flag
            Boolean save = false;
            File file = null;

            // Open
            if (e.getSource() == openFile) {
                final int returnVal = fc.showOpenDialog(GyroRotor.this);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    file = fc.getSelectedFile();
                    directory = file.getParentFile();
                    Properties prop = new Properties();
                    try {
                        prop.load(new FileInputStream(file));
                        aboutMenu.readUnitsProperties(prop);
                        atmosphereGUI.read(prop);
                        glauertRotorGUI.read(prop);
                        fileName = file;
                        fileNameGUI.setFileName(file.getName());
                    } catch (IOException ex) {
                        // Open error dialog 
                        JOptionPane.showMessageDialog(null,
                                "Error reading from file", "Open file",
                                JOptionPane.ERROR_MESSAGE);
                    }
                }
            }

            // Save
            if (e.getSource() == saveFile) {
                if (fileName == null) {
                    final int returnVal = fc.showSaveDialog(GyroRotor.this);
                    if (returnVal == JFileChooser.APPROVE_OPTION) {
                        save = true;
                        file = fc.getSelectedFile();
                        directory = file.getParentFile();
                        fileName = file;
                        fileNameGUI.setFileName(file.getName());
                    }
                } else {
                    save = true;
                    file = fileName;
                }
            }

            // Save as
            if (e.getSource() == saveFileAs) {
                final int returnVal = fc.showSaveDialog(GyroRotor.this);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    save = true;
                    file = fc.getSelectedFile();
                    directory = file.getParentFile();
                    fileName = file;
                    fileNameGUI.setFileName(file.getName());
                }
            }

            // Save datas
            if (save) {
                Properties prop = new Properties();
                aboutMenu.writeUnitsProperties(prop);
                atmosphereGUI.write(prop);
                glauertRotorGUI.write(prop);
                String fileWithExtension = file.toString();
                if (!fileWithExtension.endsWith(".rot")) {
                    fileWithExtension += ".rot";
                }
                try {
                    prop.store(new FileOutputStream(fileWithExtension), null);
                } catch (IOException ex) {
                    // Open error dialog 
                    JOptionPane.showMessageDialog(null,
                            "Error writing to file", "Save file",
                            JOptionPane.ERROR_MESSAGE);
                }
            }
        }

        @Override
        public void itemStateChanged(ItemEvent e) {
        }
    }

    /**
     * Constuctor
     */
    public GyroRotor() {

        // Decimal separator
        Locale.setDefault(new Locale("en", "US"));

        // Set layout
        setLayout(new MigLayout("fill"));

        // GyroKit object Constructors -----------------------------------------
        final ConstantAirfoil bladeProfile = new ConstantAirfoil(5.7, 0.011, 0.005, 0.278);
        final ConstantInducedVelocity inducedVelocity = new ShaydakovInducedVelocity();
        atmosphere = new DensityAtmosphere(1.225);
        final double R = 4;
        final double c = 0.2;
        final double m = 12;
        final double Ib = m * R * R / 3;
        final double Ic = m * R * R / 3 + m * c * c / 12;
        rotor = new GlauertRotor(2, R, c, 0.30, 0, 6350, 0.97, Ib, Ic,
                bladeProfile, inducedVelocity
        );

        // GUI constructor -----------------------------------------------------
        fileNameGUI = new FileNameGUI("File name : ");
        atmosphereGUI = new AtmosphereGUI(atmosphere, 0);
        glauertRotorGUI = new GlauertRotorGUI(rotor);
        rotorResultsGUI = new GlauertRotorResultsGUI();
        rotorDerivativesGUI = new RotorDerivativesGUI();
        flightData = new FlightData();
        aoaDistributionData = new AoaDistributionData();

        // Chart constructor ---------------------------------------------------
        aoaDistributionChart = new AoaDistributionChart(500);

        // Tab pane ------------------------------------------------------------
        JTabbedPane tabPane = new JTabbedPane();
        // Rotor panel
        JPanel rotorPanel = new JPanel(new MigLayout("hidemode 3,insets 5 5 5 5"));
        rotorPanel.add(fileNameGUI, "wrap");
        rotorPanel.add(atmosphereGUI, "wrap");
        rotorPanel.add(glauertRotorGUI);
        tabPane.addTab("Data", rotorPanel);
        // Simulation panel
        JPanel simulationPanel = new JPanel(new MigLayout("insets 5 5 5 5,fillx"));
        simulateButton = new JButton("Run");
        simulationPanel.add(simulateButton, "wrap");
        simulationPanel.add(flightData, "aligny top");
        simulationPanel.add(rotorResultsGUI.getCoefficientPanel(), "aligny top, wrap");
        simulationPanel.add(rotorResultsGUI.getMainPanel(), "aligny top");
        simulationPanel.add(rotorResultsGUI.getForceAndTorquePanel(), "aligny top");
        tabPane.addTab("Simulation", simulationPanel);
        // Rotor derivatives panel
        JPanel derivativesPanel = new JPanel(new MigLayout("insets 5 5 5 5,fillx"));
        derivativesButton = new JButton("Run");
        derivativesPanel.add(derivativesButton, "wrap");
        derivativesPanel.add(rotorDerivativesGUI, "aligny top");
        tabPane.addTab("Derivatives", derivativesPanel);
        // Aoa distribution panel
        JPanel aoaDistributionPanel = new JPanel(new MigLayout("insets 5 5 5 5,fillx"));
        aoaDistributionButton = new JButton("Run");
        aoaDistributionPanel.add(aoaDistributionButton, "wrap");
        aoaDistributionPanel.add(aoaDistributionData, "wrap");
        aoaDistributionPanel.add(aoaDistributionChart, "alignx center");
        tabPane.addTab("Aoa distribution", aoaDistributionPanel);

        // Add tabpane to main panel
        add(tabPane, "growx");

        // Actions -------------------------------------------------------------
        Simulate myListener = new Simulate();
        simulateButton.addActionListener(myListener);
        derivativesButton.addActionListener(myListener);
        aoaDistributionButton.addActionListener(myListener);

        // Menus ---------------------------------------------------------------
        menuBar = new JMenuBar();

        // Menu GyroRotor
        aboutMenu = new AboutMenu("GyroRotor", "3.01", "rotor performance.") {
            @Override
            public void unitsUpdate() {
                glauertRotorGUI.unitsUpdate();
                atmosphereGUI.unitsUpdate();
                rotorDerivativesGUI.unitsUpdate();
                rotorResultsGUI.unitsUpdate();
                flightData.unitsUpdate();
                frame.pack();
                frame.setMinimumSize(frame.getSize());
            }
        };
        menuBar.add(aboutMenu);

        // Menu File
        JMenu menuFile = new JMenu("File");
        menuBar.add(menuFile);
        MyItemsListener listener = new MyItemsListener();

        openFile = new JMenuItem("Open ...");
        openFile.addActionListener(listener);
        menuFile.add(openFile);
        saveFile = new JMenuItem("Save");
        saveFile.addActionListener(listener);
        menuFile.add(saveFile);
        saveFileAs = new JMenuItem("Save as ...");
        menuFile.add(saveFileAs);
        saveFileAs.addActionListener(listener);
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        // GyroRotor panel -----------------------------------------------------
        final GyroRotor gyroRotor = new GyroRotor();

        JFrame.setDefaultLookAndFeelDecorated(true);
        frame = new JFrame("GyroRotor");

        // Menus ---------------------------------------------------------------
        frame.setJMenuBar(gyroRotor.menuBar);

        // Panels --------------------------------------------------------------
        frame.setContentPane(gyroRotor);

        // Frame attributes.
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.setMinimumSize(frame.getSize());
        frame.setVisible(true);
    }
}
